/*
 *	Copyright (C) 2009 by Chebotarev Roman <roma@ultranet.ru>
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "nagiosagent.h"
#include "informationwindow.h"
#include "serviceclass.h"
#include "about.h"

const QString NagiosAgent::defaultExecDir = "/nagios/cgi-bin/";
const QString NagiosAgent::defaultHostRequest = "status.cgi?hostgroup=all&style=hostdetail&hoststatustypes=12";
const QString NagiosAgent::defaultSrvcsRequest = "status.cgi?host=all&servicestatustypes=28";
const QString NagiosAgent::defaultHttpServerPort = "80";
const QString NagiosAgent::defaultHttpsServerPort = "443";
const QString NagiosAgent::defaultAutoCloseTime = "10";
const QString NagiosAgent::defaultBasicHostsRegExp =
	"^\\s*<td .*\\s?class=['\"](status_list)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::defaultBasicSrvcsRegExp =
	"^\\s*<TD .*\\s?class=['\"](status_list)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::defaultExpertHostsRegExp =
	"^\\s*<td .*\\s?class=['\"](statusHOSTDOWN|statusHOSTUNREACHABLE)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::defaultExpertSrvcsRegExp =
	"^\\s*<TD .*\\s?class=['\"](statusBGCRITICAL)['\"]\\s?.*><a href=['\"](\\S+)['\"].*>(.+)</a>";
const QString NagiosAgent::trayToolTipHeader = "Nagios Agent";
const QString NagiosAgent::configDirName = "NetPatch";
const QString NagiosAgent::programmName = "NagiosAgent";

NagiosAgent::NagiosAgent(QWidget *parent)
    : QWidget(parent,
    		Qt::WindowSystemMenuHint | Qt::WindowMinimizeButtonHint |
				Qt::WindowMaximizeButtonHint | Qt::WindowTitleHint )
{
    QTextCodec* codec = QTextCodec::codecForName("CP1251");
    QTextCodec::setCodecForTr(codec);
    QTextCodec::setCodecForCStrings(codec);
    QTextCodec::setCodecForLocale(codec);

    ui.setupUi(this);
    //    
	QRect screenRect = QApplication::desktop()->screenGeometry(this);
	setGeometry(screenRect.width()/2 - defaultWidth/2, screenRect.height()/2 - defaultHeight/2,
				defaultWidth, defaultHeight);

	informWinPt.rx() = 100;
    informWinPt.ry() = 100;
    httpHostsErrors = 0;
	httpSrvcsErrors = 0;
	httpHostsConnectingTime = 0;
	httpSrvcsConnectingTime = 0;

	ignoredHostsSignal = true;	//  true ..      (  )
	ignoredSrvcsSignal = true;	//     QHttp     
								//   .
	authReqErrorMsgVisible = false;

	httpHosts = new QHttp;
	httpSrvcs = new QHttp;

    readConfig();
    createTrayIcon();
    connectSignals();
    createActions();

    if(runningConfig.useHttps)
    {
    	hostsSslSocket = new QSslSocket;
    	srvcsSslSocket = new QSslSocket;
    	if(runningConfig.denyInvalidCert == false)	//    
    	{
			hostsSslSocket->setPeerVerifyMode(QSslSocket::VerifyNone);
			srvcsSslSocket->setPeerVerifyMode(QSslSocket::VerifyNone);
    	}
        httpHosts->setSocket(hostsSslSocket);
    	httpSrvcs->setSocket(srvcsSslSocket);
    }
    else
    {
    	hostsHttpSocket = new QTcpSocket;
    	srvcsHttpSocket = new QTcpSocket;
    	httpHosts->setSocket(hostsHttpSocket);
    	httpSrvcs->setSocket(srvcsHttpSocket);
    }


    if(!isConfigOk())
    {
    	updateTrayToolTip(true);
    	QMessageBox::warning(this, "Nagios Agent:  .",
    			"  .    .");
    	showAndReadConfig();
    	return;
    }
	ignoredHostsSignal = false;	//  ,       QHttp 
	ignoredSrvcsSignal = false;	// 
    httpGetInfo();
    timerStart();
}


void NagiosAgent::connectSignals()
{
	connect(ui.btnAddHost, SIGNAL(clicked()), this, SLOT(addHost()));
	connect(ui.btnDelHost, SIGNAL(clicked()), this, SLOT(delHost()));
	connect(ui.btnAddService, SIGNAL(clicked()), this, SLOT(addService()));
	connect(ui.btnDelService, SIGNAL(clicked()), this, SLOT(delService()));
	connect(ui.btnSave, SIGNAL(clicked()), this, SLOT(checkAndSaveConfig()));
    connect(ui.actionQuit, SIGNAL(triggered()), qApp, SLOT(quit()));
    connect(ui.actionConfig, SIGNAL(triggered()), this, SLOT(showAndReadConfig()));
    connect(httpHosts, SIGNAL(done(bool)), this, SLOT(processHostsFinished(bool)));
    connect(httpSrvcs, SIGNAL(done(bool)), this, SLOT(processSrvcsFinished(bool)));
    connect(httpHosts, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError> &)));
    connect(httpSrvcs, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError>&)));
    connect(&timerRequests, SIGNAL(timeout()), this, SLOT(processTimer()));
    connect(&trayHostsMenu, SIGNAL(triggered(QAction *)), this, SLOT(hostsSrvcsMenuHandler(QAction *)));
    connect(&traySrvcsMenu, SIGNAL(triggered(QAction *)), this, SLOT(hostsSrvcsMenuHandler(QAction *)));
    connect(ui.actionCloseAll, SIGNAL(triggered()), this, SLOT(closeAll()));
    connect(ui.actionAbout, SIGNAL(triggered()), this, SLOT(showAbout()));
    connect(ui.cbUseHttps, SIGNAL(toggled(bool)), this, SLOT(useHttpsClicked(bool)));
    connect(&trayIcon, SIGNAL(messageClicked()), this, SLOT(trayMessageClicked()));
}

void NagiosAgent::trayMessageClicked()
{
	if(runningConfig.openUrlFromTray && lastUrlForOpenFromTrayMessage.length())
		QDesktopServices::openUrl(lastUrlForOpenFromTrayMessage);
}

void NagiosAgent::useHttpsClicked(bool use)
{
	if(use)
	{
		if( QMessageBox::information(this, "Nagios Agent",
								"         HTTPS (" +
								defaultHttpsServerPort + ")?",
								QMessageBox::Yes | QMessageBox::No,
								QMessageBox::Yes)
								 == QMessageBox::Yes)
			ui.edServerPort->setText(defaultHttpsServerPort);
	}
	else
	{
		if( QMessageBox::information(this, "Nagios Agent",
								"         HTTP (" +
								defaultHttpServerPort + ")?",
								QMessageBox::Yes | QMessageBox::No,
								QMessageBox::Yes)
								 == QMessageBox::Yes)
			ui.edServerPort->setText(defaultHttpServerPort);
	}
}

void NagiosAgent::processSslErrors(const QList<QSslError> & sslErrorsList)
{
	if(runningConfig.denyInvalidCert == false)
		return;
	static bool errorMessageVisible = false;
	if(errorMessageVisible)
		return;
	errorMessageVisible = true;

	updateTrayToolTip(true);
	timerRequests.stop();
	httpHosts->close();
	httpSrvcs->close();

	QString errors;
	int num = 0;
	Q_FOREACH(QSslError error, sslErrorsList)
		errors += tr("%1").arg(++num) + ") " + error.errorString() + "\n";
	switch(QMessageBox::critical(this, "Nagios Agent:  SSL",
			"  " + runningConfig.serverAddress + ":" + runningConfig.serverPort +
			"   :\n" + errors +
			"\n\"Yes\" -   " +
			"\n\"No\" -  .   \"\"   " +
			"\n SSL   .",
			QMessageBox::Yes | QMessageBox::No,
			QMessageBox::No))
	{
	case QMessageBox::Yes:
		qApp->quit();
		break;
	case QMessageBox::No:
		showAndReadConfig();
		ui.tabConfig->setCurrentIndex(miscTab);
		break;
	default:
		break;
	}
	errorMessageVisible = false;
}

void NagiosAgent::showHostUpOrDownMessage(QString hostName, InformationTypes state,
								QString hostHttpLink, QString hostStatus)
{
	if(runningConfig.useTrayMessages)
	{
		lastUrlForOpenFromTrayMessage = hostHttpLink;
		trayIcon.showMessage( QString("  ")
				+ ((state == hostUp || state == hostDown)? "." : ".") ,
				": " + hostName + "\n" +
				": " + hostStatus,
				(state == hostDown || state == serviceDown)? QSystemTrayIcon::Critical : QSystemTrayIcon::Information
			);
	}
	else
	{
		updateInformWinPt();
		InformationWindow *b = new InformationWindow(0, state,
				"<a href='" + hostHttpLink + "'>" + hostName +"</a>",
				hostStatus,
				runningConfig.autoCloseTime,
				informWinPt, runningConfig.popupsAlwaysStayOnTop);
		connect(this, SIGNAL(closeAllInformWin()), b, SLOT(close()));
	}
	if(!runningConfig.disableSounds)
	{
		QFile soundFile((state == hostDown || state == serviceDown)? "HostDown.wav" : "HostUp.wav");
		if(soundFile.exists())
			QSound::play(soundFile.fileName());
	}
}

void NagiosAgent::closeAll()
{
	closeAllInformWin();
}

//       
//  -  /,  
//    
void NagiosAgent::updateInformWinPt()
{
	QRect screenRect = QApplication::desktop()->screenGeometry(this);
	if(informWinPt.ry() > screenRect.height() - 220)
		informWinPt.ry() = 100;		// 
	else informWinPt.ry() += 15;
	if(informWinPt.rx() > screenRect.width() - 380)
		informWinPt.rx() = 100;		// 
	else informWinPt.rx() += 15;
}

bool NagiosAgent::isConfigOk()
{
#define CONF_ERR(msg, widget, tab) { \
		QMessageBox::warning(this, "Nagios Agent:  ", msg); \
		ui.tabConfig->setCurrentIndex(tab); \
		widget->setFocus(); \
		return false; \
	}
#define CONF_WARN(msg)	QMessageBox::information(this, "Nagios Agent: ", msg);

	if(ui.edServerAddress->text().isEmpty())
		CONF_ERR("    Nagios.", ui.edServerAddress, commonTab);
	if(ui.edServerPort->text().isEmpty())
	{
		CONF_WARN("          .")
		ui.edServerPort->setText((ui.cbUseHttps->isChecked())? defaultHttpsServerPort : defaultHttpServerPort);
	}
	else
	{
		if(ui.edServerPort->text().toInt(0, 10) < 1 ||
				ui.edServerPort->text().toInt(0, 10) > 65535)
			CONF_ERR("     Nagios.", ui.edServerPort, commonTab);
	}
	if(ui.cbHostsRequest->isChecked())
	{
		if(ui.edHostsRequest->text().isEmpty())
			CONF_ERR("      .", ui.edHostsRequest, commonTab);
	}
	if(ui.cbServicesRequest->isChecked())
	{
		if(ui.edServicesRequest->text().isEmpty())
			CONF_ERR("      .", ui.edServicesRequest, commonTab);
	}
	if(ui.cbUseHTTPAuth->isChecked())
	{
		if(ui.edLogin->text().isEmpty())
			CONF_ERR("  HTTP .", ui.edLogin, commonTab);
		if(ui.edPassword->text().isEmpty())
			CONF_ERR("  HTTP .", ui.edPassword, commonTab);
	}
	if(ui.cbHostsRegExp->isChecked())
	{
		QRegExp rx(ui.edHostsRegExp->text());
		if(!rx.isValid() || !ui.edHostsRegExp->text().length())
		{
			CONF_ERR("       .", ui.edHostsRegExp, statusTab);
		}
	}
	if(ui.cbSrvcsRegExp->isChecked())
	{
		QRegExp rx(ui.edSrvcsRegExp->text());
		if(!rx.isValid() || !ui.edSrvcsRegExp->text().length())
			CONF_ERR("       .", ui.edSrvcsRegExp, statusTab);
	}
	if(ui.rbBaseRegExpConfig->isChecked())
	{
		if(!ui.cbHostDownStat->isChecked() && !ui.cbHostUnreachStat->isChecked())
			CONF_ERR("      !", ui.frameBasicRegExpConf, statusTab);
		if(!ui.cbServiceCritStat->isChecked() && !ui.cbServiceWarnStat->isChecked() &&
				!ui.cbServiceUnknownStat->isChecked())
			CONF_ERR("      !", ui.frameBasicRegExpConf, statusTab);
	}
	if(ui.cbUseHttps->isChecked())
	{
	    if (!QSslSocket::supportsSsl())
	    	CONF_ERR("<font>" +
	    			QString("    OpenSSL.   <br>") +
	    			"   HTTPS    " +
	    			"<a href='http://www.openssl.org/related/binaries.html'></a>.<br>" +
	    			"      HTTPS .</font>",
	    			ui.cbUseHttps, commonTab);
	}
#undef CONF_ERR
#undef CONF_WARN
	return true;
}

void NagiosAgent::showAbout()
{
	About *p = new About();
	p->show();
}

void NagiosAgent::timerStart()
{
	timerRequests.start(runningConfig.requestPeriod * 1000); //      
}

void NagiosAgent::processTimer()
{
	timerRequests.setInterval(runningConfig.requestPeriod * 1000); //      
	httpGetInfo();
}

void NagiosAgent::processHostsInfo()
{
	QAction * tmpAction;
	QStringList newHosts;	//       ,
							//        
							//    .
	bool foundInHostsList = false;
	QString tmp = httpHosts->readAll();
	//     
	/*QFile file("status.cgi.htm");
	file.open(QIODevice::ReadOnly );
	tmp = file.readAll();*/

	QStringList httpStrings = tmp.split("\n", QString::SkipEmptyParts);		//  HTML    
	QRegExp rx(runningConfig.hostsRegExp);
	rx.setCaseSensitivity(Qt::CaseInsensitive);
	//  HTTP ,      
	//  trayHostsMenu  Action   text -   ,
	//  data       
    Q_FOREACH(QString str,  httpStrings)
    {
    	if(rx.indexIn(str) > -1)	//  
    	{
    		//        
    		foundInHostsList = false;
    		Q_FOREACH(QString host, runningConfig.hostsList)
    		{
    			if(host.toLower() == rx.cap(runningConfig.hostNameRxPos).toLower())
				{
					foundInHostsList = true;
					break;
				}
    		}
    		if(foundInHostsList && runningConfig.allExceptSpecifedHosts)
    			continue;
    		else if(!foundInHostsList && runningConfig.onlyListedHosts)
    				continue;
			//          
			newHosts << rx.cap(runningConfig.hostNameRxPos);
    		//        
    		Q_FOREACH(QAction * action, trayHostsMenu.actions())
    		{
    			if(action == ui.actionHostsDownList)
    				continue;
    			//     -     
       			if(action->text().toLower() == rx.cap(runningConfig.hostNameRxPos).toLower())
    				goto next_string;
    		}

    		//   
			trayHostsMenu.removeAction(ui.actionEmpty);
			tmpAction = new QAction(this);
			tmpAction->setText(rx.cap(runningConfig.hostNameRxPos));
			tmpAction->setData(runningConfig.serverAddress + runningConfig.execDir + rx.cap(runningConfig.hostLinkRxPos));
			trayHostsMenu.addAction(tmpAction);
			if(runningConfig.showHostsStateChanged)
				showHostUpOrDownMessage(rx.cap(runningConfig.hostNameRxPos), hostDown,
					((runningConfig.useHttps)? "https://" : "http://") + tmpAction->data().toString(),
					rx.cap(runningConfig.hostStatRxPos).remove("status"));
			//hostsStateChanged = true;
next_string:;
	    }
    }
    //     ( trayHostsMenu)  
	//   .    - 
	//   -       
    bool ignore = false;
	Q_FOREACH(QAction * action, trayHostsMenu.actions())
	{
		if(action == ui.actionHostsDownList)
			continue;
		if(!action->data().toBool())
			continue;
		Q_FOREACH(QString host, newHosts)
		{
			if(host.toLower() == action->text().toLower())
				goto next_action;
		}
		// -  -     
		if(runningConfig.allExceptSpecifedHosts)
		{
			ignore = false;
			Q_FOREACH(QString host, runningConfig.hostsList)
			{
				if(host.toLower() == action->text().toLower())
				{
					ignore = true;
					break;
				}
			}
		}else
		{
			ignore = true;
			Q_FOREACH(QString host, runningConfig.hostsList)
			{
				if(host.toLower() == action->text().toLower())
				{
					ignore = false;
					break;
				}
			}
		}
		if(!ignore && runningConfig.showHostsStateChanged)
			showHostUpOrDownMessage(action->text(), hostUp,
				((runningConfig.useHttps)? "https://" : "http://") + action->data().toString(), "Host UP");
		trayHostsMenu.removeAction(action);
		delete action;
		//hostsStateChanged = true;
next_action:;
	}

    if(trayHostsMenu.actions().count() < 3)
    	trayHostsMenu.addAction(ui.actionEmpty);
    //      
    updateTrayToolTip();
}

void NagiosAgent::processSrvcsFinished(bool error)
{
	if(!runningConfig.enableSrvcs)
		return;

	if(ignoredSrvcsSignal)	//    ,     
	{						//       
		ignoredSrvcsSignal = false;
	    return;
	}

	if(error)
	{
		QString errTitle = "Nagios Agent:  HTTP   ";
		switch(httpSrvcs->error())
		{
		case QHttp::HostNotFound:
		case QHttp::ConnectionRefused:
		case QHttp::UnexpectedClose:
		case QHttp::InvalidResponseHeader:
		case QHttp::WrongContentLength:
		case QHttp::ProxyAuthenticationRequiredError:
			++httpSrvcsErrors;
			if(httpSrvcsErrors <= maxSrvcsErrors)
				break;
			if( QMessageBox::critical(this, errTitle,
									"     :\n" +
									QString("'") + httpHosts->errorString() + QString("'\n") +
									"  ?",
									QMessageBox::Yes | QMessageBox::No,
									QMessageBox::No)
									 == QMessageBox::Yes)
				qApp->exit(1);
		    break;
		case QHttp::AuthenticationRequiredError:
			if(authReqErrorMsgVisible)
				return;
			authReqErrorMsgVisible = true;
			timerRequests.stop();
			if( QMessageBox::critical(this, errTitle,
								"    .   ? "
								"  \"No\"   ",
								QMessageBox::Yes | QMessageBox::No,
								QMessageBox::No) == QMessageBox::Yes)
				qApp->exit(2);
			else
			{
				showAndReadConfig();
				ui.edLogin->setFocus();
			}
			authReqErrorMsgVisible = false;
			break;
		case QHttp::Aborted:
		    	break;
		case QHttp::UnknownError:
		default:
			++httpSrvcsErrors;
			if(httpSrvcsErrors <= maxSrvcsErrors)
				break;
			if (
					QMessageBox::critical(this, errTitle,
									"      .\n"
									"  ?",
									QMessageBox::Yes | QMessageBox::No, QMessageBox::No)
									== QMessageBox::Yes)
				qApp->exit(1);
			break;
		}
		return;
	}
	int httpStatus = 0;
	httpStatus = httpSrvcs->lastResponse().statusCode();
	if( httpStatus != httpOk)
	{
		QString errTitle = "Nagios Agent:  HTTP   ";
		timerRequests.stop();
		switch  (
				QMessageBox::warning(this, errTitle,
				"   : "
				" '" + runningConfig.serverAddress + "'   " + tr("%1").arg(httpStatus) + "\n"
				"  ? ( Cancel   )",
				QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel,
				QMessageBox::Cancel))
		{
			case QMessageBox::Yes:
				qApp->quit();
				break;
			case QMessageBox::No:
				timerStart();
				break;
			case QMessageBox::Cancel:
				showAndReadConfig();
				break;
			default:
				break;
		}
		return;
	}
	httpSrvcsErrors = 0;
	processSrvcsInfo();
}

void NagiosAgent::processHostsFinished(bool error)
{
	if(!runningConfig.enableHosts)
		return;

	if(ignoredHostsSignal)	//    ,     
	{						//       
	    ignoredHostsSignal = false;
	    return;
	}

	if(error)
	{
		QString errTitle = "Nagios Agent:  HTTP   ";
		switch(httpHosts->error())
		{
		case QHttp::HostNotFound:
		case QHttp::ConnectionRefused:
		case QHttp::UnexpectedClose:
		case QHttp::InvalidResponseHeader:
		case QHttp::WrongContentLength:
		case QHttp::ProxyAuthenticationRequiredError:
			++httpHostsErrors;
			if(httpHostsErrors <= maxHostsErrors)
				break;
			if( QMessageBox::critical(this, errTitle,
									"     :\n" +
									QString("'") + httpHosts->errorString() + QString("'\n") +
									"  ?",
									QMessageBox::Yes | QMessageBox::No,
									QMessageBox::No)
									 == QMessageBox::Yes)
				qApp->exit(1);
		    break;
		case QHttp::AuthenticationRequiredError:
			if(authReqErrorMsgVisible)
				break;
			authReqErrorMsgVisible = true;
			timerRequests.stop();
			if( QMessageBox::critical(this, errTitle,
								"    .   ? "
								"  \"No\"   ",
								QMessageBox::Yes | QMessageBox::No,
								QMessageBox::No) == QMessageBox::Yes)
				qApp->exit(2);
			else
			{
				showAndReadConfig();
				ui.edLogin->setFocus();
			}
			authReqErrorMsgVisible = false;
			break;
		case QHttp::Aborted:
		    	break;
		case QHttp::UnknownError:
		default:
			++httpHostsErrors;
			if(httpHostsErrors <= maxHostsErrors)
				break;
			if (
					QMessageBox::critical(this, errTitle,
									"      .\n"
									"  ?",
									QMessageBox::Yes | QMessageBox::No, QMessageBox::No)
									== QMessageBox::Yes)
				qApp->exit(1);
			break;
		}
		return;
	}
	int httpStatus = httpHosts->lastResponse().statusCode();
	if( httpStatus != httpOk)
	{
		QString errTitle = "Nagios Agent:  HTTP   ";
		timerRequests.stop();
		switch  (
				QMessageBox::warning(this, errTitle,
				"   : "
				" '" + runningConfig.serverAddress + "'   " + tr("%1").arg(httpStatus) + "\n"
				"  ? ( Cancel   )",
				QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel,
				QMessageBox::Cancel))
		{
			case QMessageBox::Yes:
				qApp->quit();
				break;
			case QMessageBox::No:
				timerStart();
				break;
			case QMessageBox::Cancel:
				showAndReadConfig();
				break;
			default:
				break;
		}
		return;
	}
	httpHostsErrors = 0;
	processHostsInfo();
}


void NagiosAgent::processSrvcsInfo()
{
	QAction * tmpAction;
	QStringList newSrvcs;	//       ,
							//        
							//    .
	QString hostSrvc;
	bool foundInSrvcsList = false;
	QString tmp = httpSrvcs->readAll();
	//     
	/*QFile file("status.cgi.htm");
	file.open(QIODevice::ReadOnly );
	tmp = file.readAll();*/

	QStringList httpStrings = tmp.split("\n", QString::SkipEmptyParts);

	QRegExp rx(runningConfig.srvcsRegExp);	//      
	QRegExp rxHostName(
			"host=(\\S+)[&$]"	//          
	);

	rx.setCaseSensitivity(Qt::CaseInsensitive);
	//  HTTP ,      
	//  traySrvcsMenu  Action   text -   ,
	//  data       
    Q_FOREACH(QString str,  httpStrings)
    {
    	if(rx.indexIn(str) > -1)
    	{

    		if(rxHostName.indexIn(rx.cap(runningConfig.srvLinkRxPos)) < 0)	//       
    			goto next_string;											//    -   
    		//        
    		foundInSrvcsList = false;
    		Q_FOREACH(ServiceClass service, runningConfig.srvcsList)
    		{
    			if(service.isHostFound(rx.cap(runningConfig.srvNameRxPos), rxHostName.cap(1)))
    			{
    				foundInSrvcsList = true;
    				break;
    			}
    		}
    		if(foundInSrvcsList && runningConfig.allExceptSpecifedSrvcs)
    			continue;
    		else if(!foundInSrvcsList && runningConfig.onlyListedSrvcs)
    				continue;
			//          
    		hostSrvc = rx.cap(runningConfig.srvNameRxPos) + ":" + rxHostName.cap(1);
			newSrvcs << hostSrvc ;
    		//        
    		Q_FOREACH(QAction * action, traySrvcsMenu.actions())
    		{
    			if(action == ui.actionSrvcsProblemsList)
    				continue;
    			if(action->text().toLower() == hostSrvc.toLower())
    				goto next_string;	//  -      
    		}

    		//   
			traySrvcsMenu.removeAction(ui.actionEmpty);
			tmpAction = new QAction(this);
			tmpAction->setText(hostSrvc);
			tmpAction->setData(runningConfig.serverAddress + runningConfig.execDir + rx.cap(runningConfig.srvLinkRxPos));
			traySrvcsMenu.addAction(tmpAction);
			if(runningConfig.showSrvcsStateChanged)
				showHostUpOrDownMessage(hostSrvc, serviceDown,
					((runningConfig.useHttps)? "https://" : "http://") + tmpAction->data().toString(),
					rx.cap(runningConfig.srvStatRxPos).remove("statusBG"));
			//srvcsStateChanged = true;
next_string:;
	    }
    }
    //     ( traySrvcsMenu)  
	//   .    - 
	//   -       
    bool ignore = false;
	QString hostName;
	QString currSrvc;
	Q_FOREACH(QAction * action, traySrvcsMenu.actions())
	{
		if(action == ui.actionSrvcsProblemsList)
			continue;
		// -   :    
		//    
		if(!action->data().toBool())
			continue;
		Q_FOREACH(QString service, newSrvcs)
		{
			if(service.toLower() == action->text().toLower())
				goto next_action;	//        
		}
		// -  -     
		currSrvc = action->text().split(":").first();
		hostName = action->text().split(":").last();
		if(runningConfig.allExceptSpecifedSrvcs)
		{
			ignore = false;
    		Q_FOREACH(ServiceClass service, runningConfig.srvcsList)
    		{
    			if(service.isHostFound(currSrvc, hostName))
    			{
    				ignore = true;
    				break;
    			}
    		}
		}else
		{
			ignore = true;
    		Q_FOREACH(ServiceClass service, runningConfig.srvcsList)
    		{
    			if(service.isHostFound(currSrvc, hostName))
    			{
    				ignore = false;
    				break;
    			}
    		}
		}
		if(!ignore && runningConfig.showSrvcsStateChanged)
			showHostUpOrDownMessage(action->text(), serviceUp,
				((runningConfig.useHttps)? "https://" : "http://") + action->data().toString(), "Service Up");
		traySrvcsMenu.removeAction(action);
		delete action;
next_action:;
	}

    if(traySrvcsMenu.actions().count() < 3)
    	traySrvcsMenu.addAction(ui.actionEmpty);
    //      
    updateTrayToolTip();
}

void NagiosAgent::updateTrayToolTip(bool reset)
{
	QString newToolTip = trayToolTipHeader;
	if(runningConfig.serverAddress.length())
		newToolTip += QString(" (") + ((runningConfig.useHttps)? "https://" : "http://") +
			runningConfig.serverAddress + ":" + runningConfig.serverPort + ")";

	if(reset)
	{
		newToolTip += "\n  ...";
		trayIcon.setToolTip(newToolTip);
		return;
	}
	if(runningConfig.enableHosts)
	{
			if((trayHostsMenu.actions().count() > 2) &&
					(trayHostsMenu.actions().last() != ui.actionEmpty))
				newToolTip += "\n : " + tr("%1").arg(trayHostsMenu.actions().count() - 2);
	}
	else
		newToolTip += "\n  ";
	if(runningConfig.enableSrvcs)
	{
			if((traySrvcsMenu.actions().count() > 2) &&
					(traySrvcsMenu.actions().last() != ui.actionEmpty))
				newToolTip += "\n : " + tr("%1").arg(traySrvcsMenu.actions().count() - 2);
	}
	else
		newToolTip += "\n  ";
	newToolTip += "\n : " + QDateTime::currentDateTime().toString("hh:mm:ss dd.MM.yyyy");
	trayIcon.setToolTip(newToolTip);
}

void NagiosAgent::httpGetInfo()
{
	if(runningConfig.useHTTPAuth)
	{
		httpHosts->setUser(runningConfig.httpLogin, runningConfig.httpPassword);
		httpSrvcs->setUser(runningConfig.httpLogin, runningConfig.httpPassword);
	}else
	{
		httpHosts->setUser("", "");
		httpSrvcs->setUser("", "");
	}
	if(runningConfig.enableHosts)
	{	//   http .
		if(httpHosts->state() == QHttp::Connecting)
		{	// QHttp     Connecting   
			//   ,    
			//    .
			httpHostsConnectingTime += runningConfig.requestPeriod;
			if(httpHostsConnectingTime >= maxHttpConnectingTime)	//   
			{														//  .
				timerRequests.stop();
				httpHosts->abort();
				if(QMessageBox::critical(this, "Nagios Agent:  HTTP", "     "
						"   .\n"
						"  ?", QMessageBox::Yes | QMessageBox::No,
						QMessageBox::No) == QMessageBox::Yes)
					qApp->quit();
				httpHostsConnectingTime = 0;	//   No -    
				timerStart();	//  .
			}
		}
		else
		{	// QHttp     Connecting -    
			//   .
			httpHostsConnectingTime = 0;
			httpHosts->setHost(runningConfig.serverAddress,
							(runningConfig.useHttps)? QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp,
							runningConfig.serverPort.toInt(0, 10));
			idGetHosts = httpHosts->get(runningConfig.execDir + "/" + runningConfig.hostsRequest);
		}
	}
	if(runningConfig.enableSrvcs)
	{	//. ,   httpHosts
		if(httpSrvcs->state() == QHttp::Connecting)
		{
			httpSrvcsConnectingTime += runningConfig.requestPeriod;
			if(httpSrvcsConnectingTime >= maxHttpConnectingTime)
			{
				timerRequests.stop();
				httpSrvcs->abort();
				if(QMessageBox::critical(this, "Nagios Agent:  HTTP", "     "
						"   .\n"
						"  ?", QMessageBox::Yes | QMessageBox::No,
						QMessageBox::No) == QMessageBox::Yes)
					qApp->quit();
				httpSrvcsConnectingTime = 0;
				timerStart();
			}
		}
		else
		{
			httpSrvcsConnectingTime = 0;
			httpSrvcs->setHost(runningConfig.serverAddress,
							(runningConfig.useHttps)? QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp,
							runningConfig.serverPort.toInt(0, 10));
			idGetSrvcs = httpSrvcs->get(runningConfig.execDir + "/" + runningConfig.servicesRequest);
		}
	}
}

void NagiosAgent::showAndReadConfig()
{
	//     
	if(isVisible())
	{
		this->activateWindow();
		return;
	}
	QRect screenRect = QApplication::desktop()->screenGeometry(this);
	setGeometry(screenRect.width()/2 - defaultWidth/2, screenRect.height()/2 - defaultHeight/2,
				defaultWidth, defaultHeight);
	ui.tabConfig->setCurrentIndex(commonTab);
	readConfig();
	show();
}

void NagiosAgent::checkAndSaveConfig()
{
	if(!isConfigOk())
		return;
	saveConfig();
	timerStart();
	hide();
}

void NagiosAgent::saveConfig()
{
	QSettings config(configDirName, programmName);

	config.setValue("ServerAddress", ui.edServerAddress->text());
	if(ui.cbUseHttps->isChecked())
		config.setValue("HTTPSPort", ui.edServerPort->text());
	else
		config.setValue("HTTPPort", ui.edServerPort->text());
	config.setValue("DenySSLCert", ui.rbDenyInvalidCert->isChecked());
	config.setValue("RequestPeriod", ui.spinBoxPeriod->value());
	config.setValue("AutoClose", ui.cbAutoClose->isChecked());
	config.setValue("UseUserDefExecDir", ui.cbExecDir->isChecked());
	config.setValue("UserDefNagiosExecDir", ui.edExecDir->text());
	config.setValue("UseUserDefHostsRequest", ui.cbHostsRequest->isChecked());
	config.setValue("UseUserDefSrvcsRequest", ui.cbServicesRequest->isChecked());
	config.setValue("UseHTTPAuth", ui.cbUseHTTPAuth->isChecked());
	config.setValue("AutoCloseTime", ui.spinBoxClose->value());
	config.setValue("Login", ui.edLogin->text());
	config.setValue("Password", ui.edPassword->text());
	config.setValue("UserDefHostsRequest", ui.edHostsRequest->text());
	config.setValue("UserDefServicesRequest", ui.edServicesRequest->text());
	config.setValue("AllExceptSpecifedHosts", ui.rbAllExceptSpecifedHosts->isChecked());
	config.setValue("OnlyListedHosts", ui.rbOnlyListedHosts->isChecked());
	config.setValue("AllExceptSpecifedSrvcs", ui.rbAllExceptSpecifedSrvcs->isChecked());
	config.setValue("OnlyListedSrvcs", ui.rbOnlyListedSrvcs->isChecked());
	config.setValue("UseTrayMessages", ui.cbUseTrayMessages->isChecked());
	config.setValue("DisableSounds", ui.cbDisableSounds->isChecked());
	config.setValue("EnableHostsMonitoring", ui.cbEnableHosts->isChecked());
	config.setValue("EnableSrvcsMonitoring", ui.cbEnableSrvcs->isChecked());
	config.setValue("ShowHostsMessages", ui.cbShowHosts->isChecked());
	config.setValue("ShowSrvcsMessages", ui.cbShowSrvcs->isChecked());
	config.setValue("UseUserDefHostsRegExp", ui.cbHostsRegExp->isChecked());
	config.setValue("HostsRegExp", ui.edHostsRegExp->text());
	config.setValue("UseUserDefSrvcsRegExp", ui.cbSrvcsRegExp->isChecked());
	config.setValue("SrvcsRegExp", ui.edSrvcsRegExp->text());
	config.setValue("HostStatRxPos", ui.edHostStatRxPos->text());
	config.setValue("HostLinkRxPos", ui.edHostLinkRxPos->text());
	config.setValue("HostNameRxPos", ui.edHostNameRxPos->text());
	config.setValue("SrvStatRxPos", ui.edSrvStatRxPos->text());
	config.setValue("SrvLinkRxPos", ui.edSrvLinkRxPos->text());
	config.setValue("SrvNameRxPos", ui.edSrvNameRxPos->text());
	config.setValue("UseHTTPS", ui.cbUseHttps->isChecked());
	config.setValue("PopupsStayOnTop", ui.cbPopupsStayOnTop->isChecked());
	config.setValue("UseBasicRegExpConfig", ui.rbBaseRegExpConfig->isChecked());
	config.setValue("UseHostDownStat", ui.cbHostDownStat->isChecked());
	config.setValue("UseHostUnreachStat",ui.cbHostUnreachStat->isChecked());
	config.setValue("UseSrvCritStat", ui.cbServiceCritStat->isChecked());
	config.setValue("UseSrvWarnStat", ui.cbServiceWarnStat->isChecked());
	config.setValue("UseSrvUnknStat", ui.cbServiceUnknownStat->isChecked());
	config.setValue("OpenUrlFromTray", ui.cbOpenUrlFromTray->isChecked());

	if(ui.rbHostOnLMK->isChecked())
		config.setValue("NumMenuOnLMK", hostsMenu);
	else if(ui.rbSrvcsOnLMK->isChecked())
		config.setValue("NumMenuOnLMK", srvcsMenu);
	else
		config.setValue("NumMenuOnLMK", containerMenu);


	QString string = "";
	for(int i = 0; i < ui.listHosts->count(); ++i)
		string += ui.listHosts->item(i)->text() + "|";
	config.setValue("HostsList", string);
	string.clear();
	for(int i = 0; i < ui.listServices->count(); ++i)
		string += ui.listServices->item(i)->text() + "|";
	config.setValue("ServicesList", string);

	updateRunningConfig();
	updateTrayMenus();
	updateTrayToolTip();
	httpGetInfo();
	timerStart();
}

void NagiosAgent::updateTrayMenus()
{
	if(!runningConfig.enableHosts) 	//    -  
	{								//   
		Q_FOREACH(QAction * action, trayHostsMenu.actions())
		{
			if((!action->data().toBool() || action == ui.actionSrvcsProblemsList) &&
				action != ui.actionEmpty)
				continue;
			trayHostsMenu.removeAction(action);
			if(action != ui.actionEmpty)
				delete action;
		}
		trayHostsMenu.addAction(ui.actionUnmonitored);
	}
	else
	{
		trayHostsMenu.removeAction(ui.actionUnmonitored);
		if(trayHostsMenu.actions().count() < 3)
			trayHostsMenu.addAction(ui.actionEmpty);
	}
	if(!runningConfig.enableSrvcs) 	//    -  
	{								//   
		Q_FOREACH(QAction * action, traySrvcsMenu.actions())
		{
			if((!action->data().toBool() || action == ui.actionSrvcsProblemsList) &&
				action != ui.actionEmpty)
				continue;
			traySrvcsMenu.removeAction(action);
			if(action != ui.actionEmpty)
				delete action;
		}
		traySrvcsMenu.addAction(ui.actionUnmonitored);
	}
	else
	{
		traySrvcsMenu.removeAction(ui.actionUnmonitored);
		if(traySrvcsMenu.actions().count() < 3)
			traySrvcsMenu.addAction(ui.actionEmpty);
	}
}

void NagiosAgent::reinitQHttp(bool httpsUsed, bool useHttps)
{
	disconnect(httpHosts, SIGNAL(done(bool)), this, SLOT(processHostsFinished(bool)));
    disconnect(httpSrvcs, SIGNAL(done(bool)), this, SLOT(processSrvcsFinished(bool)));
    disconnect(httpHosts, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError> &)));
    disconnect(httpSrvcs, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError>&)));

    ignoredHostsSignal = true;
    ignoredSrvcsSignal = true;

    timerRequests.stop();

    httpHosts->close();
	httpSrvcs->close();

    delete httpHosts;
	delete httpSrvcs;

    httpHosts = new QHttp;
    httpSrvcs = new QHttp;

    if(httpsUsed) //  HTTPS
	{
		delete hostsSslSocket;
		delete srvcsSslSocket;
	}
	else	//  HTTP
	{
		delete hostsHttpSocket;
		delete srvcsHttpSocket;
	}

    if(useHttps)
    {
    	hostsSslSocket = new QSslSocket;
    	srvcsSslSocket = new QSslSocket;
    	if(runningConfig.denyInvalidCert == false)	//    
    	{
    		hostsSslSocket->setPeerVerifyMode(QSslSocket::VerifyNone);
    		srvcsSslSocket->setPeerVerifyMode(QSslSocket::VerifyNone);
    	}
        httpHosts->setSocket(hostsSslSocket);
    	httpSrvcs->setSocket(srvcsSslSocket);
    }
    else
    {
		hostsHttpSocket = new QTcpSocket;
		srvcsHttpSocket = new QTcpSocket;
		httpHosts->setSocket(hostsHttpSocket);
		httpSrvcs->setSocket(srvcsHttpSocket);
    }

    connect(httpHosts, SIGNAL(done(bool)), this, SLOT(processHostsFinished(bool)));
    connect(httpSrvcs, SIGNAL(done(bool)), this, SLOT(processSrvcsFinished(bool)));
    connect(httpHosts, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError> &)));
    connect(httpSrvcs, SIGNAL(sslErrors (const QList<QSslError> &)), this, SLOT(processSslErrors(const QList<QSslError>&)));

	timerStart();
}

void NagiosAgent::updateBasicRegExp()
{
	// 
	QString resultExpr = "";
	QString resultRegExp = defaultBasicHostsRegExp;
	if(ui.cbHostDownStat->isChecked())
		resultExpr = "statusHOSTDOWN";
	if(ui.cbHostUnreachStat->isChecked())
		resultExpr += QString((resultExpr.length())? "|" : "") + "statusHOSTUNREACHABLE";
	resultRegExp.replace("status_list", resultExpr);
	runningConfig.hostsRegExp = resultRegExp;
	runningConfig.hostStatRxPos = defaultHostStatRxPos;
	runningConfig.hostLinkRxPos = defaultHostLinkRxPos;
	runningConfig.hostNameRxPos = defaultHostNameRxPos;
	// 
	resultExpr.clear();
	resultRegExp = defaultBasicSrvcsRegExp;
	if(ui.cbServiceCritStat->isChecked())
		resultExpr = "statusBGCRITICAL";
	if(ui.cbServiceWarnStat->isChecked())
		resultExpr += QString((resultExpr.length())? "|" : "") + "statusBGWARNING";
	if(ui.cbServiceUnknownStat->isChecked())
		resultExpr += QString((resultExpr.length())? "|" : "") + "statusBGUNKNOWN";
	resultRegExp.replace("status_list", resultExpr);
	runningConfig.srvcsRegExp = resultRegExp;
	runningConfig.srvStatRxPos = defaultSrvStatRxPos;
	runningConfig.srvLinkRxPos = defaultSrvLinkRxPos;
	runningConfig.srvNameRxPos = defaultSrvNameRxPos;
}

void NagiosAgent::updateRunningConfig()
{
	runningConfig.serverAddress = ui.edServerAddress->text();
	runningConfig.serverPort = ui.edServerPort->text();
	runningConfig.requestPeriod = ui.spinBoxPeriod->value();
	runningConfig.autoClose = ui.cbAutoClose->isChecked();
	runningConfig.useUserDefExecDir = ui.cbExecDir->isChecked();
	runningConfig.useUserDefHostRequest = ui.cbHostsRequest->isChecked();
	runningConfig.useUserDefSrvcsRequest = ui.cbServicesRequest->isChecked();
	runningConfig.useHTTPAuth = ui.cbUseHTTPAuth->isChecked();
	if(runningConfig.autoClose)
		runningConfig.autoCloseTime = ui.spinBoxClose->value();
	else runningConfig.autoCloseTime = 0;
	runningConfig.httpLogin = ui.edLogin->text();
	runningConfig.httpPassword = ui.edPassword->text();
	runningConfig.allExceptSpecifedHosts = ui.rbAllExceptSpecifedHosts->isChecked();
	runningConfig.onlyListedHosts = ui.rbOnlyListedHosts->isChecked();
	runningConfig.allExceptSpecifedSrvcs = ui.rbAllExceptSpecifedSrvcs->isChecked();
	runningConfig.onlyListedSrvcs = ui.rbOnlyListedSrvcs->isChecked();
	runningConfig.useTrayMessages = ui.cbUseTrayMessages->isChecked();
	runningConfig.disableSounds = ui.cbDisableSounds->isChecked();
	runningConfig.enableHosts = ui.cbEnableHosts->isChecked();
	runningConfig.popupsAlwaysStayOnTop = ui.cbPopupsStayOnTop->isChecked();
	runningConfig.useBasicRegExpConfig = ui.rbBaseRegExpConfig->isChecked();
	runningConfig.showHostsStateChanged = ui.cbShowHosts->isChecked();
	runningConfig.showSrvcsStateChanged = ui.cbShowSrvcs->isChecked();
	runningConfig.useUserDefHostsRegExp = ui.cbHostsRegExp->isChecked();
	runningConfig.useUserDefSrvcsRegExp = ui.cbSrvcsRegExp->isChecked();
	runningConfig.openUrlFromTray = ui.cbOpenUrlFromTray->isChecked();

	if(ui.rbHostOnLMK->isChecked())
		runningConfig.menuNumOnLMK = hostsMenu;
	else if(ui.rbSrvcsOnLMK->isChecked())
		runningConfig.menuNumOnLMK = srvcsMenu;
	else
		runningConfig.menuNumOnLMK = containerMenu;

	//     SSL 
	if(runningConfig.denyInvalidCert != ui.rbDenyInvalidCert->isChecked())
	{
		runningConfig.denyInvalidCert = ui.rbDenyInvalidCert->isChecked();
		reinitQHttp(runningConfig.useHttps, runningConfig.useHttps);
	}
	//  
	if(runningConfig.useHttps != ui.cbUseHttps->isChecked())
	{
		reinitQHttp(runningConfig.useHttps, ui.cbUseHttps->isChecked());
		runningConfig.useHttps = ui.cbUseHttps->isChecked();
	}

	if(!runningConfig.enableHosts)
		idGetHosts = 0;
	runningConfig.enableSrvcs = ui.cbEnableSrvcs->isChecked();
	if(!runningConfig.enableSrvcs)
		idGetSrvcs = 0;

	if(runningConfig.useBasicRegExpConfig)
		updateBasicRegExp();
	else
	{
		if(runningConfig.useUserDefHostsRegExp)
		{
			runningConfig.hostsRegExp = ui.edHostsRegExp->text();
			runningConfig.hostStatRxPos = ui.edHostStatRxPos->text().toInt();
			runningConfig.hostLinkRxPos = ui.edHostLinkRxPos->text().toInt();
			runningConfig.hostNameRxPos = ui.edHostNameRxPos->text().toInt();
		}
		else
		{
			runningConfig.hostsRegExp = defaultExpertHostsRegExp;
			runningConfig.hostStatRxPos = defaultHostStatRxPos;
			runningConfig.hostLinkRxPos = defaultHostLinkRxPos;
			runningConfig.hostNameRxPos = defaultHostNameRxPos;
		}
		if(runningConfig.useUserDefSrvcsRegExp)
		{
			runningConfig.srvcsRegExp = ui.edSrvcsRegExp->text();
			runningConfig.srvStatRxPos = ui.edSrvStatRxPos->text().toInt();
			runningConfig.srvLinkRxPos = ui.edSrvLinkRxPos->text().toInt();
			runningConfig.srvNameRxPos = ui.edSrvNameRxPos->text().toInt();
		}
		else
		{
			runningConfig.srvcsRegExp = defaultExpertSrvcsRegExp;
			runningConfig.srvStatRxPos = defaultSrvStatRxPos;
			runningConfig.srvLinkRxPos = defaultSrvLinkRxPos;
			runningConfig.srvNameRxPos = defaultSrvNameRxPos;
		}
	}

	if(runningConfig.useUserDefExecDir)
		runningConfig.execDir = ui.edExecDir->text();
	else runningConfig.execDir = defaultExecDir;
	if(runningConfig.useUserDefHostRequest)
		runningConfig.hostsRequest = ui.edHostsRequest->text();
	else runningConfig.hostsRequest = defaultHostRequest;
	if(runningConfig.useUserDefSrvcsRequest)
		runningConfig.servicesRequest = ui.edServicesRequest->text();
	else runningConfig.servicesRequest = defaultSrvcsRequest;

	runningConfig.hostsList.clear();
	for(int i = 0; i < ui.listHosts->count(); ++i)
		runningConfig.hostsList.append(ui.listHosts->item(i)->text());
	runningConfig.srvcsList.clear();
	for(int i = 0; i < ui.listServices->count(); ++i)
		runningConfig.srvcsList.append(ui.listServices->item(i)->text());
}

void NagiosAgent::readConfig()
{
	ui.listHosts->clear();
	ui.listServices->clear();
	QSettings config(configDirName, programmName);

	runningConfig.serverAddress = config.value("ServerAddress", "").toString();
	runningConfig.useHttps = config.value("UseHTTPS", defaultUseHttps).toBool();
	if(runningConfig.useHttps)
		runningConfig.serverPort = config.value("HTTPSPort", defaultHttpsServerPort).toString();
	else
		runningConfig.serverPort = config.value("HTTPPort", defaultHttpServerPort).toString();
	runningConfig.denyInvalidCert = config.value("DenySSLCert", defaultDenyInvalidCert).toBool();
	runningConfig.requestPeriod = config.value("RequestPeriod", defaultRequestPeriod).toInt();
	runningConfig.useHTTPAuth = config.value("UseHTTPAuth", useHTTPAuth).toBool();
	runningConfig.autoClose = config.value("AutoClose", defaultAutoClose).toBool();
	if(runningConfig.autoClose)
		runningConfig.autoCloseTime = config.value("AutoCloseTime", defaultAutoCloseTime).toInt();
	else runningConfig.autoCloseTime = 0;
	runningConfig.httpLogin = config.value("Login", "").toString();
	runningConfig.httpPassword = config.value("Password", "").toString();
	runningConfig.useUserDefHostRequest = config.value("UseUserDefHostsRequest", useUserDefHostsRequest).toBool();
	runningConfig.useUserDefExecDir = config.value("UseUserDefExecDir", useUserDefExecDir).toBool();
	if(runningConfig.useUserDefExecDir)
		runningConfig.execDir = config.value("UserDefNagiosExecDir", defaultExecDir).toString();
	else runningConfig.execDir = defaultExecDir;
	if(runningConfig.useUserDefHostRequest)
		runningConfig.hostsRequest = config.value("UserDefHostsRequest", defaultHostRequest).toString();
	else runningConfig.hostsRequest = defaultHostRequest;
	runningConfig.useUserDefSrvcsRequest = config.value("UseUserDefSrvcsRequest", useUserDefSrvcsRequest).toBool();
	if(runningConfig.useUserDefSrvcsRequest)
		runningConfig.servicesRequest = config.value("UserDefServicesRequest", defaultSrvcsRequest).toString();
	else runningConfig.servicesRequest = defaultSrvcsRequest;
	runningConfig.allExceptSpecifedHosts = config.value("AllExceptSpecifedHosts", defaultAllExceptSpecifedHosts).toBool();
	runningConfig.onlyListedHosts = config.value("OnlyListedHosts", defaultOnlyListedHosts).toBool();
	runningConfig.allExceptSpecifedSrvcs = config.value("AllExceptSpecifedSrvcs", defaultAllExceptSpecifedSrvcs).toBool();
	runningConfig.onlyListedSrvcs = config.value("OnlyListedSrvcs", defaultOnlyListedSrvcs).toBool();
	runningConfig.hostsList = config.value("HostsList", "").toString().split("|", QString::SkipEmptyParts);
	runningConfig.srvcsList = config.value("ServicesList", "").toString().split("|", QString::SkipEmptyParts);
	runningConfig.useTrayMessages = config.value("UseTrayMessages", false).toBool();
	runningConfig.disableSounds = config.value("DisableSounds", false).toBool();
	runningConfig.enableHosts = config.value("EnableHostsMonitoring", true).toBool();
	runningConfig.enableSrvcs = config.value("EnableSrvcsMonitoring", true).toBool();
	runningConfig.showHostsStateChanged = config.value("ShowHostsMessages", true).toBool();
	runningConfig.showSrvcsStateChanged = config.value("ShowSrvcsMessages", true).toBool();
	runningConfig.useUserDefHostsRegExp = config.value("UseUserDefHostsRegExp", false).toBool();
	runningConfig.useUserDefSrvcsRegExp = config.value("UseUserDefSrvcsRegExp", false).toBool();
	runningConfig.popupsAlwaysStayOnTop = config.value("PopupsStayOnTop", defaultPopupsStayOnTop).toBool();
	runningConfig.useBasicRegExpConfig = config.value("UseBasicRegExpConfig", defaultUseBasicRegExpConfig).toBool();
	runningConfig.openUrlFromTray = config.value("OpenUrlFromTray", defaultOpenUrlFromTray).toBool();
	runningConfig.menuNumOnLMK = config.value("NumMenuOnLMK", defaultMenuNumOnLMK).toInt();

	ui.cbHostDownStat->setChecked(config.value("UseHostDownStat", defaultUseHostDownStat).toBool());
	ui.cbHostUnreachStat->setChecked(config.value("UseHostUnreachStat",defaultUseHostUnreachStat).toBool());
	ui.cbServiceCritStat->setChecked(config.value("UseSrvCritStat", defaultUseSrvCritStat ).toBool());
	ui.cbServiceWarnStat->setChecked(config.value("UseSrvWarnStat", defaultUseSrvWarnStat).toBool());
	ui.cbServiceUnknownStat->setChecked(config.value("UseSrvUnknStat", defaultUseSrvUnknStat).toBool());

	if(runningConfig.useBasicRegExpConfig)
		updateBasicRegExp();
	else
	{
		if(runningConfig.useUserDefHostsRegExp)
		{
			runningConfig.hostsRegExp = config.value("HostsRegExp", defaultExpertHostsRegExp).toString();
			runningConfig.hostStatRxPos = config.value("HostStatRxPos", defaultHostStatRxPos).toInt();
			runningConfig.hostLinkRxPos = config.value("HostLinkRxPos", defaultHostLinkRxPos).toInt();
			runningConfig.hostNameRxPos = config.value("HostNameRxPos", defaultHostNameRxPos).toInt();
		}
		else
		{
			runningConfig.hostsRegExp = defaultExpertHostsRegExp;
			runningConfig.hostStatRxPos = defaultHostStatRxPos;
			runningConfig.hostLinkRxPos = defaultHostLinkRxPos;
			runningConfig.hostNameRxPos = defaultHostNameRxPos;
		}
		if(runningConfig.useUserDefSrvcsRegExp)
		{
			runningConfig.srvcsRegExp = config.value("SrvcsRegExp", defaultExpertSrvcsRegExp).toString();
			runningConfig.srvStatRxPos = config.value("SrvStatRxPos", defaultSrvStatRxPos).toInt();
			runningConfig.srvLinkRxPos = config.value("SrvLinkRxPos", defaultSrvLinkRxPos).toInt();
			runningConfig.srvNameRxPos = config.value("SrvNameRxPos", defaultSrvNameRxPos).toInt();
		}
		else
		{
			runningConfig.srvcsRegExp = defaultExpertSrvcsRegExp;
			runningConfig.srvStatRxPos = defaultSrvStatRxPos;
			runningConfig.srvLinkRxPos = defaultSrvLinkRxPos;
			runningConfig.srvNameRxPos = defaultSrvNameRxPos;
		}
	}

	ui.edServerAddress->setText(runningConfig.serverAddress);
    ui.edServerPort->setText(runningConfig.serverPort);
    ui.spinBoxPeriod->setValue(runningConfig.requestPeriod);
	ui.cbAutoClose->setChecked(runningConfig.autoClose);
	ui.cbHostsRequest->setChecked(runningConfig.useUserDefHostRequest);
	ui.cbServicesRequest->setChecked(runningConfig.useUserDefSrvcsRequest);
	ui.cbUseHTTPAuth->setChecked(runningConfig.useHTTPAuth);
    ui.spinBoxClose->setValue(config.value("AutoCloseTime", defaultAutoCloseTime).toInt());
    ui.edLogin->setText(runningConfig.httpLogin);
    ui.edPassword->setText(runningConfig.httpPassword);
    ui.cbExecDir->setChecked(runningConfig.useUserDefExecDir);
    ui.edExecDir->setText(config.value("UserDefNagiosExecDir", defaultExecDir).toString());
    ui.edHostsRequest->setText(config.value("UserDefHostsRequest", defaultHostRequest).toString());
    ui.edServicesRequest->setText(config.value("UserDefServicesRequest", defaultSrvcsRequest).toString());
    ui.rbAllExceptSpecifedHosts->setChecked(runningConfig.allExceptSpecifedHosts);
    ui.rbOnlyListedHosts->setChecked(runningConfig.onlyListedHosts);
    ui.rbAllExceptSpecifedSrvcs->setChecked(runningConfig.allExceptSpecifedSrvcs);
    ui.rbOnlyListedSrvcs->setChecked(runningConfig.onlyListedSrvcs);
    ui.cbUseTrayMessages->setChecked(runningConfig.useTrayMessages);
    ui.cbDisableSounds->setChecked(runningConfig.disableSounds);
    ui.cbEnableHosts->setChecked(runningConfig.enableHosts);
    ui.cbEnableSrvcs->setChecked(runningConfig.enableSrvcs);
    ui.cbShowHosts->setChecked(runningConfig.showHostsStateChanged);
    ui.cbShowSrvcs->setChecked(runningConfig.showSrvcsStateChanged);
    ui.cbShowHosts->setEnabled(runningConfig.enableHosts);
    ui.cbShowSrvcs->setEnabled(runningConfig.enableSrvcs);
    ui.cbHostsRegExp->setChecked(runningConfig.useUserDefHostsRegExp);
    ui.edHostsRegExp->setText(config.value("HostsRegExp", defaultExpertHostsRegExp).toString());
    ui.cbSrvcsRegExp->setChecked(runningConfig.useUserDefSrvcsRegExp);
    ui.edSrvcsRegExp->setText(config.value("SrvcsRegExp", defaultExpertSrvcsRegExp).toString());
    ui.edHostStatRxPos->setText(tr("%1").arg(runningConfig.hostStatRxPos));
	ui.edHostLinkRxPos->setText(tr("%1").arg(runningConfig.hostLinkRxPos));
	ui.edHostNameRxPos->setText(tr("%1").arg(runningConfig.hostNameRxPos));
	ui.edSrvStatRxPos->setText(tr("%1").arg(runningConfig.srvStatRxPos));
	ui.edSrvLinkRxPos->setText(tr("%1").arg(runningConfig.srvLinkRxPos));
	ui.edSrvNameRxPos->setText(tr("%1").arg(runningConfig.srvNameRxPos));
	ui.cbUseHttps->setChecked(runningConfig.useHttps);
	ui.cbPopupsStayOnTop->setChecked(runningConfig.popupsAlwaysStayOnTop);
	ui.cbOpenUrlFromTray->setChecked(runningConfig.openUrlFromTray);

	switch(runningConfig.menuNumOnLMK)
	{
	case hostsMenu:
		ui.rbHostOnLMK->setChecked(true);
		break;
	case srvcsMenu:
		ui.rbSrvcsOnLMK->setChecked(true);
		break;
	case containerMenu:
		ui.rbHostsAndSrvcsOnLMK->setChecked(true);
		break;
	default:
		ui.rbHostOnLMK->setChecked(true);
		break;
	}
	if(runningConfig.denyInvalidCert)
		ui.rbDenyInvalidCert->setChecked(true);
	else
		ui.rbAllowInvalidCert->setChecked(true);
	if(runningConfig.useBasicRegExpConfig)
	{
		ui.rbBaseRegExpConfig->setChecked(true);
		ui.frameBasicRegExpConf->setEnabled(true);
		ui.frameExpertRegExpConf->setDisabled(true);
	}
	else
	{
		ui.rbExpertRegExpConfig->setChecked(true);
		ui.frameExpertRegExpConf->setEnabled(true);
		ui.frameBasicRegExpConf->setDisabled(true);
	}

	Q_FOREACH(QString s,  runningConfig.hostsList)
    	ui.listHosts->addItem(s);
    Q_FOREACH(QString s, runningConfig.srvcsList)
    	ui.listServices->addItem(s);
}

void NagiosAgent::createActions()
{
    ui.actionCancel->setShortcut(Qt::Key_Escape);
    ui.btnCancel->addAction(ui.actionCancel);
}

void NagiosAgent::hostsSrvcsMenuHandler(QAction * action)
{
	if(action == ui.actionHostsDownList)
		QDesktopServices::openUrl(((runningConfig.useHttps)? "https://" : "http://") +
				runningConfig.serverAddress +
				runningConfig.execDir +
				runningConfig.hostsRequest);
	else if(action == ui.actionSrvcsProblemsList)
		QDesktopServices::openUrl(((runningConfig.useHttps)? "https://" : "http://") +
				runningConfig.serverAddress +
				runningConfig.execDir +
				runningConfig.servicesRequest);

	if(!action->data().toBool())
		return;
	QDesktopServices::openUrl(((runningConfig.useHttps)? "https://" : "http://") + action->data().toString());
}

void NagiosAgent::createTrayIcon()
{
    //Create main menu
    trayMainMenu.addAction(ui.actionConfig);
    trayMainMenu.addAction(ui.actionCloseAll);
    trayMainMenu.addSeparator();
    trayMainMenu.addAction(ui.actionAbout);
    trayMainMenu.addSeparator();
    trayMainMenu.addAction(ui.actionQuit);

    ui.actionEmpty->setDisabled(true);
    ui.actionUnmonitored->setDisabled(true);
    //Create hosts menu
    trayHostsMenu.addAction(ui.actionHostsDownList);
    trayHostsMenu.addSeparator();
    trayHostsMenu.addAction((runningConfig.enableHosts)? ui.actionEmpty : ui.actionUnmonitored);

    //Create servvices menu
    traySrvcsMenu.addAction(ui.actionSrvcsProblemsList);
    traySrvcsMenu.addSeparator();
    traySrvcsMenu.addAction((runningConfig.enableSrvcs)? ui.actionEmpty : ui.actionUnmonitored);


	if (!QSystemTrayIcon::isSystemTrayAvailable())
	{
        QMessageBox::critical(0, QObject::tr("Systray"),
                              QObject::tr("I couldn't detect any system tray "
                                          "on this system."));
        exit(-1);
    }
	trayIcon.setIcon(QIcon(":/icons/icon16x16.png"));
	trayIcon.setToolTip("NagiosAgent");
	trayIcon.show();
	trayIcon.setContextMenu(&trayMainMenu);
	connect(&trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
	            this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
}

void NagiosAgent::closeEvent(QCloseEvent * event)
{
	hide();
	event->ignore();
}

void NagiosAgent::iconActivated(QSystemTrayIcon::ActivationReason reason)
{
	switch (reason)
	{
	case QSystemTrayIcon::Trigger:
		activateWindow();
		if(runningConfig.menuNumOnLMK == hostsMenu)
			trayHostsMenu.exec(QCursor::pos());
		else if(runningConfig.menuNumOnLMK == srvcsMenu)
			traySrvcsMenu.exec(QCursor::pos());
		else
		{
			trayHostsAndSrvcsMenu.clear();
			trayHostsAndSrvcsMenu.addActions(trayHostsMenu.actions());
			trayHostsAndSrvcsMenu.addSeparator();
			trayHostsAndSrvcsMenu.addActions(traySrvcsMenu.actions());
			trayHostsAndSrvcsMenu.exec(QCursor::pos());
		}
		break;
	case QSystemTrayIcon::MiddleClick:
		activateWindow();
		if(runningConfig.menuNumOnLMK == srvcsMenu)
			trayHostsMenu.exec(QCursor::pos());
		else if(runningConfig.menuNumOnLMK == hostsMenu)
			traySrvcsMenu.exec(QCursor::pos());
		break;
	case QSystemTrayIcon::Unknown:

		break;
	default:

    	break;
	}
}

void NagiosAgent::addService()
{
	if(!ui.edAddService->text().length())
		return;
	ui.listServices->addItem(ui.edAddService->text());
	ui.edAddService->clear();
}

void NagiosAgent::delService()
{
	qDeleteAll(ui.listServices->selectedItems());
}

void NagiosAgent::addHost()
{
	if(!ui.edAddHost->text().length())
		return;
	ui.listHosts->addItem(ui.edAddHost->text());
	ui.edAddHost->clear();
}

void NagiosAgent::delHost()
{
	qDeleteAll(ui.listHosts->selectedItems());
}

NagiosAgent::~NagiosAgent()
{
	timerRequests.stop();
	httpHosts->abort();
	httpSrvcs->abort();
}
